pub const Point = packed struct {
    x: u16,
    y: u16,
    pub fn init(x: u16, y: u16) @This() {
        return @This() {
            .x = x,
            .y = y,
        };
    }

    pub fn equals(this: @This(), other: @This()) bool {
        return this.x == other.x and this.y == other.y;
    }
};

pub const Rect = packed struct {
    base: Point,
    width: u8,
    height: u8,
    pub fn init(base: Point, width: u8, height: u8) @This() {
        return @This() {
            .base = base,
            .width = width,
            .height = height,
        };
    }

    pub fn inside(this: @This(), point: Point) bool {
        return point.x >= this.base.x and point.x < this.base.x + this.width and point.y >= this.base.y and point.y < this.base.y + this.height;
    }

    pub fn collides(this: @This(), other: @This()) bool {
        return this.base.x < other.base.x + other.width and this.base.x + this.width > other.base.x and this.base.y < other.base.y + other.height and this.base.y + this.height > other.base.y;
    }
};

pub const Direction = enum(u2) {
    Up,
    Down,
    Left,
    Right,

    pub fn opposite(this: @This()) @This() {
        switch(this) {
            .Up => .Down,
            .Down => .Up,
            .Left => .Right,
            .Right => .Left,
        }
    }
};

pub const Trigger = struct {
    aura: Rect,
    direction: Direction,
    dialogue: []const u8,
};

pub const Tileset = enum(u2) {
    Cave,
    Rpg,
    Dungeon,

    pub fn width(this: @This()) u9 {
        switch (this) {
            .Cave => return 28,
            .Rpg => return 34,
            .Dungeon => return 14,
        }
    }
    
    pub fn height(this: @This()) u9 {
        switch (this) {
            .Cave => return 10,
            .Rpg => return 16,
            .Dungeon => return 13,
        }
    }
};

pub const State = enum {
    Title,
    StoryDump,
    Gameplay,
};

pub const Map = struct {
    width: u8,
    height: u8,
    tileset: Tileset = .Rpg,
    tiles: []const u9,
    coll: []const Rect,
    triggers: []const Trigger,
    start: Point,
};